#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.groups.balinese_saka import BalineseSakaCalendarHolidays
from holidays.groups.buddhist import BuddhistCalendarHolidays
from holidays.groups.burmese import BurmeseCalendarHolidays
from holidays.groups.chinese import ChineseCalendarHolidays
from holidays.groups.christian import ChristianHolidays
from holidays.groups.custom import StaticHolidays
from holidays.groups.eastern import EasternCalendarHolidays
from holidays.groups.hebrew import HebrewCalendarHolidays
from holidays.groups.hindu import HinduCalendarHolidays
from holidays.groups.international import InternationalHolidays
from holidays.groups.islamic import IslamicHolidays
from holidays.groups.mandaean import MandaeanHolidays
from holidays.groups.mongolian import MongolianCalendarHolidays
from holidays.groups.persian import PersianCalendarHolidays
from holidays.groups.sinhala import SinhalaCalendarHolidays
from holidays.groups.thai import ThaiCalendarHolidays
from holidays.groups.tibetan import TibetanCalendarHolidays
